#!/usr/bin/env python3
#
# Tiny webservice to host password protected cfg files placed in /cfgdir inside a container
#

import os
import re
import sys
import logging
from io import BytesIO
import configparser
from flask import Flask, send_file
from flask_httpauth import HTTPBasicAuth
from werkzeug.security import generate_password_hash, check_password_hash
from werkzeug.exceptions import Forbidden, NotFound

app = Flask(__name__)
auth = HTTPBasicAuth()
log = logging.getLogger('globconfd')
passwords, paths = None, None
root_dir = os.environ.get('ENVUSERROOT', '/configs')
if app.debug or os.environ.get('DEBUG', False):
    logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
    log.setLevel(logging.DEBUG)
    app.debug = True
else:
    logging.basicConfig(stream=sys.stdout, level=logging.INFO)
    log.setLevel(logging.INFO)

log.info('hosting files from %s', root_dir)


@auth.verify_password
def verify_password(username, password):
    if username in passwords and check_password_hash(passwords.get(username), password):
        log.debug('Validated password of %s', username)
        return username
    log.debug('Could not validate password of user %s', username)
    return False


@app.route('/', defaults={'path': ''})
@app.route('/<path:path>')
@auth.login_required
def index(path):
    log.info('GET %s from %s', path, auth.current_user())
    path_regex = paths[auth.current_user()]
    if not re.match(path_regex, path):
        raise Forbidden('{} does not have access to read {} - paths must match {}'.format(auth.current_user(),
                                                                                          path, path_regex))
    abs_path = os.path.join(root_dir, path)
    try:
        f = open(abs_path, 'rb')
        bs = BytesIO(b'')
        bs.write(f.read())
        f.close()
        bs.seek(0)
        log.debug('Was able to read file %s for %s - sending it now', abs_path, auth.current_user())
        name = os.path.basename(abs_path)
        return send_file(filename_or_fp=bs, mimetype='text/plain', attachment_filename=name, as_attachment=True)
    except FileNotFoundError as fnfe:
        log.error('Was unable to locate file %s for %s', abs_path, auth.current_user())
        raise NotFound('No file {} exists?'.format(path))
    except PermissionError as pe:
        log.error('Was not allowed to read file %s for %s', abs_path, auth.current_user())
        raise Forbidden('The file in {} is not allowed to be read by this service? Check permissions'.format(path))


def read_users():
    global passwords, paths
    if passwords is not None:
        return
    users_cfg = configparser.ConfigParser()
    file = os.environ.get('ENVUSERCFGFILE', '/env_users.conf')
    log.info('Reading users from %s', file)
    users_cfg.read(file)
    passwords = {}
    paths = {}
    for section in users_cfg.sections():
        user = users_cfg.get(section, 'username')
        passwords[user] = generate_password_hash(users_cfg.get(section, 'password'))
        paths[user] = r'{}'.format(users_cfg.get(section, 'path_regex'))
        log.debug('Initialized user %s with path regex %s', user, paths[user])


read_users()

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000, debug=app.debug)



from unittest import TestCase
from configparser import NoSectionError, NoOptionError


class TestGlobconf(TestCase):
    def test_verify_required_options(self):
        from globconf import verify_required_options, read_config
        read_config('someapp.ini')
        try:
            cfg = verify_required_options('dummy cfg for app_user', ['test'])
            self.assertEqual(cfg['test'], 'test')
        except Exception as e:
            self.fail(str(e))
        try:
            cfg = verify_required_options('dummy cfg for app_user', ['absent'])
            self.fail('Was able to get SectionProxy for absent option?')
        except NoOptionError as noe:
            pass  # expected..

    def test_fetch_config(self):
        from globconf import verify_required_options, fetch_config
        from requests.exceptions import HTTPError, ConnectionError
        from time import time, sleep
        t1 = time()
        fetch_config('http://localhost:5000/tests/someapp.ini', 'app_user', 'app_pass', cache_timeout=1)
        t2 = time()
        fetch_config('http://localhost:5000/tests/someapp.ini', 'app_user', 'app_pass', cache_timeout=1, force=True)
        t3 = time()
        self.assertGreater(t2-t1, t3-t2, msg='Subsequent fetch of the config, after caching was slower?')
        print('Noncached: {}, cached: {}'.format(t2-t1, t3-t2))
        try:
            cfg = verify_required_options('dummy cfg for app_user', ['test'])
            self.assertEqual(cfg['test'], 'test')
        except Exception as e:
            self.fail(str(e))
        try:
            fetch_config('http://blah.com:5000/tests/someapp.ini', 'app_user', 'app_pass', cache_timeout=1, force=True)
            self.fail('Survived fetch of nonexistent host')
        except ConnectionError as he:
            pass
        sleep(1)
        try:
            fetch_config('http://localhost:5000/tests/bogus.ini', 'app_user', 'app_pass', cache_timeout=1, force=True)
            self.fail('Survived fetch of nonexistent url')
        except HTTPError as he:
            self.assertEqual(he.response.status_code, 404)
        sleep(2)
        try:
            fetch_config('http://localhost:5000/tests/someapp.ini', 'crapp_user', 'crapp_pass', cache_timeout=1, force=True)
            self.fail('Survived fetch using invalid auth')
        except HTTPError as he:
            self.assertEqual(he.response.status_code, 401)
        sleep(1)
        try:
            fetch_config('http://localhost:5000/tests/someapp.ini', 'build_agent', 'build_pass', cache_timeout=1, force=True)
            self.fail('Survived fetch using unauthed path')
        except HTTPError as he:
            self.assertEqual(he.response.status_code, 403)







import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="globconf",
    version="0.0.5",
    author="Steffen Schumacher",
    author_email="ssch@wheel.dk",
    description="global configparser object to be used across modules",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/steffenschumacher/globconf",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    install_requires=['requests', 'diskcache'],
    setup_requires=['pytest-runner', 'wheel', 'twine'],
)



from configparser import ConfigParser, NoOptionError, NoSectionError, SectionProxy

"""
global/singleton config object to be uses across modules
"""
config = None


def verify_required_options(section, option_keys):
    """
    Verifies that section exists, and that it has option_keys defined
    :param str section: Section in the config
    :param list[str] option_keys: list of required options
    :rtype: SectionProxy
    :raises: NoSectionError, NoOptionError
    """
    myconfig = read_config()
    if section not in myconfig:
        raise NoSectionError(section)
    for option in option_keys:
        if option not in myconfig[section]:
            raise NoOptionError(option, section)
    return myconfig[section]


def read_config(path='config.ini', force=False):
    """
    :param str path:
    :param bool force:
    :rtype: ConfigParser
    """
    global config
    if not config or force:
        config = ConfigParser()
        config.read(path)
    return config


def fetch_config(url, user, password, cache_timeout=86400, force=False):
    """
    :param str url:
    :param str user:
    :param str password:
    :param int cache_timeout: timeout seconds
    :param bool force: force config reload
    :rtype: ConfigParser
    """
    global config
    if not config or force:
        import os
        import hashlib
        from diskcache import Cache
        from requests import get
        from requests.auth import HTTPBasicAuth
        url_hash = hashlib.md5('{}{}{}'.format(url, user, password).encode()).hexdigest()
        cfg_path = os.environ.get('GLOBCONF_CACHE_PATH', 'tmp')
        config = ConfigParser()

        with Cache(cfg_path, timeout=cache_timeout) as c:
            cfg_string, expire = c.get(url_hash, expire_time=True)
            if cfg_string is None:
                r = get(url, auth=HTTPBasicAuth(user, password), timeout=5)
                r.raise_for_status()
                cfg_string = r.text
                c.add(url_hash, cfg_string)
        config.read_string(cfg_string)
    return config
